@@ -19,16 +19,15 @@ sudo apt install\
        	dvipng\
        	librsvg2-bin\
    	imagemagick\
-	inkscape\
+	chromium-browser\
 	libcanberra-gtk-module\
        	docbook2x\
 	graphviz\
 	gfortran\
+	dbus\
 	#
 
 sudo apt install\
-	python3.6\
-	python3.6-venv\
 	python3.7\
 	python3.7-venv\
 	python3.9\
