@@ -5,8 +5,7 @@ from subprocess import check_call
 
 def main(version, prevversion, outdir):
     check_version(version, outdir)
-    run_stage(['bin/mailmap_update.py'])
-    run_stage(['bin/authors_update.py'])
+    run_stage(['bin/mailmap_check.py', '--update-authors'])
     run_stage(['mkdir', '-p', outdir])
     build_release_files('bdist_wheel', 'sympy-%s-py3-none-any.whl', outdir, version)
     build_release_files('sdist', 'sympy-%s.tar.gz', outdir, version)
