@@ -47,6 +47,8 @@ pip install\
   sphinx\
   sphinx-math-dollar\
   sphinx-reredirects\
+  myst-parser\
+  linkify-it-py\
   #
 
 pip freeze > requirements.txt
